@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # coding: utf-8
 
 """
@@ -52,7 +51,7 @@ XL_CAN_RXMSG_FLAG_EF = 0x0200
 XL_CAN_STD = 1
 XL_CAN_EXT = 2
 
-XLuint64 = ctypes.c_ulonglong
+XLuint64 = ctypes.c_int64
 XLaccess = XLuint64
 XLhandle = ctypes.c_void_p
 
@@ -64,6 +63,7 @@ XL_CAN_MAX_DATA_LEN = 64
 XL_INTERFACE_VERSION = 3
 XL_INTERFACE_VERSION_V4 = 4
 
+XL_BUS_ACTIVE_CAP_CAN = XL_BUS_TYPE_CAN << 16
 XL_CHANNEL_FLAG_CANFD_ISO_SUPPORT = 0x80000000
 
 # structure for XL_RECEIVE_MSG, XL_TRANSMIT_MSG
@@ -145,6 +145,55 @@ class XLcanFdConf(ctypes.Structure):
                 ('tseg1Dbr', ctypes.c_uint), ('tseg2Dbr', ctypes.c_uint),
                 ('reserved', ctypes.c_uint * 2)]
 
+class XLchannelConfig(ctypes.Structure):
+    _pack_ = 1
+    _fields_ = [
+        ('name', ctypes.c_char * 32),
+        ('hwType', ctypes.c_ubyte),
+        ('hwIndex', ctypes.c_ubyte),
+        ('hwChannel', ctypes.c_ubyte),
+        ('transceiverType', ctypes.c_ushort),
+        ('transceiverState', ctypes.c_ushort),
+        ('configError', ctypes.c_ushort),
+        ('channelIndex', ctypes.c_ubyte),
+        ('channelMask', XLuint64),
+        ('channelCapabilities', ctypes.c_uint),
+        ('channelBusCapabilities', ctypes.c_uint),
+        ('isOnBus', ctypes.c_ubyte),
+        ('connectedBusType', ctypes.c_uint),
+        ('busParams', ctypes.c_ubyte * 32),
+        ('_doNotUse', ctypes.c_uint),
+        ('driverVersion', ctypes.c_uint),
+        ('interfaceVersion', ctypes.c_uint),
+        ('raw_data', ctypes.c_uint * 10),
+        ('serialNumber', ctypes.c_uint),
+        ('articleNumber', ctypes.c_uint),
+        ('transceiverName', ctypes.c_char * 32),
+        ('specialCabFlags', ctypes.c_uint),
+        ('dominantTimeout', ctypes.c_uint),
+        ('dominantRecessiveDelay', ctypes.c_ubyte),
+        ('recessiveDominantDelay', ctypes.c_ubyte),
+        ('connectionInfo', ctypes.c_ubyte),
+        ('currentlyAvailableTimestamps', ctypes.c_ubyte),
+        ('minimalSupplyVoltage', ctypes.c_ushort),
+        ('maximalSupplyVoltage', ctypes.c_ushort),
+        ('maximalBaudrate', ctypes.c_uint),
+        ('fpgaCoreCapabilities', ctypes.c_ubyte),
+        ('specialDeviceStatus', ctypes.c_ubyte),
+        ('channelBusActiveCapabilities', ctypes.c_ushort),
+        ('breakOffset', ctypes.c_ushort),
+        ('delimiterOffset', ctypes.c_ushort),
+        ('reserved', ctypes.c_uint * 3)
+    ]
+
+class XLdriverConfig(ctypes.Structure):
+    _fields_ = [
+        ('dllVersion', ctypes.c_uint),
+        ('channelCount', ctypes.c_uint),
+        ('reserved', ctypes.c_uint * 10),
+        ('channel', XLchannelConfig * 64)
+    ]
+
 # driver status
 XLstatus = ctypes.c_short
 
@@ -159,6 +208,11 @@ def check_status(result, function, arguments):
     return result
 
 
+xlGetDriverConfig = _xlapi_dll.xlGetDriverConfig
+xlGetDriverConfig.argtypes = [ctypes.POINTER(XLdriverConfig)]
+xlGetDriverConfig.restype = XLstatus
+xlGetDriverConfig.errcheck = check_status
+
 xlOpenDriver = _xlapi_dll.xlOpenDriver
 xlOpenDriver.argtypes = []
 xlOpenDriver.restype = XLstatus
