@@ -66,6 +66,13 @@ CAN_MSG_STRUCT = struct.Struct("<HBBL8s")
 # valid data bytes, data
 CAN_FD_MSG_STRUCT = struct.Struct("<HBBLLBBB5x64s")
 
+# channel, dlc, valid payload length of data, tx count, arbitration id,
+# frame length, flags, bit rate used in arbitration phase,
+# bit rate used in data phase, time offset of brs field,
+# time offset of crc delimiter field, bit count, direction,
+# offset if extDataOffset is used, crc
+CAN_FD_MSG_64_STRUCT = struct.Struct("<BBBBLLLLLLLHBBL")
+
 # channel, length, flags, ecc, position, dlc, frame length, id, flags ext, data
 CAN_ERROR_EXT_STRUCT = struct.Struct("<HHLBBBxLLH2x8s")
 
@@ -81,6 +88,7 @@ CAN_ERROR_EXT = 73
 CAN_MESSAGE2 = 86
 GLOBAL_MARKER = 96
 CAN_FD_MESSAGE = 100
+CAN_FD_MESSAGE_64 = 101
 
 NO_COMPRESSION = 0
 ZLIB_DEFLATE = 2
@@ -91,6 +99,12 @@ EDL = 0x1
 BRS = 0x2
 ESI = 0x4
 
+# CAN FD 64 Flags
+REMOTE_FLAG_64 = 0x0010
+EDL_64 = 0x1000
+BRS_64 = 0x2000
+ESI_64 = 0x4000
+
 TIME_TEN_MICS = 0x00000001
 TIME_ONE_NANS = 0x00000002
 
@@ -181,8 +195,11 @@ class BLFReader(BaseIOHandler):
                         raise BLFParseError()
 
                     obj_size = header[3]
+                    obj_type = header[4]
                     # Calculate position of next object
-                    next_pos = pos + obj_size + (obj_size % 4)
+                    next_pos = pos + obj_size
+                    if obj_type != CAN_FD_MESSAGE_64:
+                        next_pos += obj_size % 4
                     if next_pos > len(data):
                         # Object continues in next log container
                         break
@@ -208,7 +225,6 @@ class BLFReader(BaseIOHandler):
                         factor = 1e-9
                     timestamp = timestamp * factor + self.start_timestamp
 
-                    obj_type = header[4]
                     # Both CAN message types have the same starting content
                     if obj_type in (CAN_MESSAGE, CAN_MESSAGE2):
                         (channel, flags, dlc, can_id,
@@ -236,6 +252,29 @@ class BLFReader(BaseIOHandler):
                                       data=can_data[:length],
                                       channel=channel - 1)
                         yield msg
+                    elif obj_type == CAN_FD_MESSAGE_64:
+                        (
+                            channel, dlc, _, _, can_id, _, fd_flags
+                         ) = CAN_FD_MSG_64_STRUCT.unpack_from(data, pos)[:7]
+                        length = dlc2len(dlc)
+                        can_data = struct.unpack_from(
+                            "<{}s".format(length),
+                            data,
+                            pos + CAN_FD_MSG_64_STRUCT.size
+                        )[0]
+                        msg = Message(
+                            timestamp=timestamp,
+                            arbitration_id=can_id & 0x1FFFFFFF,
+                            is_extended_id=bool(can_id & CAN_MSG_EXT),
+                            is_remote_frame=bool(fd_flags & REMOTE_FLAG_64),
+                            is_fd=bool(fd_flags & EDL_64),
+                            bitrate_switch=bool(fd_flags & BRS_64),
+                            error_state_indicator=bool(fd_flags & ESI_64),
+                            dlc=length,
+                            data=can_data[:length],
+                            channel=channel - 1
+                        )
+                        yield msg
                     elif obj_type == CAN_ERROR_EXT:
                         (channel, _, _, _, _, dlc, _, can_id, _,
                          can_data) = CAN_ERROR_EXT_STRUCT.unpack_from(data, pos)
@@ -247,6 +286,8 @@ class BLFReader(BaseIOHandler):
                                       data=can_data[:dlc],
                                       channel=channel - 1)
                         yield msg
+                    # else:
+                    #     LOG.warning("Unknown object type (%d)", obj_type)
 
                     pos = next_pos
 
