@@ -29,8 +29,8 @@ class Message(object):
 
     :func:`~copy.copy`/:func:`~copy.deepcopy` is supported as well.
 
-    Messages do not support "dynamic" attributes, meaning any others that the
-    documented ones.
+    Messages do not support "dynamic" attributes, meaning any others than the
+    documented ones, since it uses :attr:`~object.__slots__`.
     """
 
     __slots__ = (
@@ -53,7 +53,9 @@ class Message(object):
         # TODO keep this for a version, in order to not break old code
         # this entire method (as well as the _dict attribute in __slots__ and the __setattr__ method)
         # can be removed in 4.0
-        # this method is only called if the attribute was not found elsewhere, like in __slots__
+        # this method is only called if the attribute was not found elsewhere, like in __slots_
+        if key not in self.__slots__:
+            raise AttributeError
         try:
             warnings.warn("Custom attributes of messages are deprecated and will be removed in 4.0", DeprecationWarning)
             return self._dict[key]
@@ -110,6 +112,7 @@ class Message(object):
         if is_extended_id is not None:
             self.is_extended_id = is_extended_id
         else:
+            # Default behaviour is to create extended id messages
             self.is_extended_id = True if extended_id is None else extended_id
 
         self.is_remote_frame = is_remote_frame
@@ -303,7 +306,7 @@ class Message(object):
             if self.bitrate_switch:
                 raise ValueError("bitrate switch is only allowed for CAN FD frames")
             if self.error_state_indicator:
-                raise ValueError("error stat indicator is only allowed for CAN FD frames")
+                raise ValueError("error state indicator is only allowed for CAN FD frames")
 
     def equals(self, other, timestamp_delta=1.0e-6):
         """
